/*  _____ _
 * |_   _| |_  _ _ ___ ___ _ __  __ _
 *   | | | ' \| '_/ -_) -_) '  \/ _` |_
 *   |_| |_||_|_| \___\___|_|_|_\__,_(_)
 *
 * Threema for Android
 * Copyright (c) 2023-2025 Threema GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

package ch.threema.app.multidevice.wizard.steps

import android.animation.LayoutTransition
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.view.updatePadding
import ch.threema.app.R
import ch.threema.app.utils.getStatusBarHeightPxCompat
import ch.threema.app.utils.withCurrentWindowInsets

class LinkNewDeviceLinkingProgressFragment : LinkNewDeviceFragment() {
    private lateinit var bodyTextView: TextView

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?,
    ): View? {
        return inflater.inflate(R.layout.fragment_link_new_device_progress, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        bodyTextView = view.findViewById(R.id.body)
        view.findViewById<LinearLayout>(R.id.explain_layout).layoutTransition.enableTransitionType(
            LayoutTransition.CHANGING,
        )

        // Add extra vertical top padding to prevent ui elements being overlapped
        // by the system status bar when the bottom-sheet is fully expanded
        withCurrentWindowInsets { _, insets ->
            view.findViewById<ConstraintLayout>(R.id.parent_layout).updatePadding(
                top = insets.getStatusBarHeightPxCompat(),
            )
        }

        viewModel.linkingProgressStringRes.observe(viewLifecycleOwner) { stringRes ->
            bodyTextView.text = getString(stringRes)
        }
    }
}
